/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule auto-download-history-dialog-view
 */

(function() {
var AutoDownloadHistoryDialogView;
var prototype;
var superConstructor;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var ModalDialogView = dixim.common.ModalDialogView;
var escape = dixim.xml.escape;
var Configs = dixim.ctt.Configs;
var $ = dixim.ctt.jQuery;

/**
 * @class AutoDownloadHistoryDialogView
 * @constructor
 * @param {Object} configs
 */
AutoDownloadHistoryDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(AutoDownloadHistoryDialogView, ModalDialogView);
dixim.autoDownload.AutoDownloadHistoryDialogView = AutoDownloadHistoryDialogView;
prototype = AutoDownloadHistoryDialogView.prototype;
superConstructor = AutoDownloadHistoryDialogView.superConstructor;

var HISTORY_DELETE = 'historyDelete';

var HISTORY_IMAGES = {
    success: [Configs.get('imagePath'), '/ico_st_ok.png' ].join(''),
    error: [Configs.get('imagePath'), '/ico_st_error.png'].join(''),
};
var TEMPLATE_BODY_CONTENT = [
    '<div class="ctt-transfer-history-view auto-download-history-view dlg_scroll">',
    '<div id="auto-download-history-no-content">', '</div>',
    '<table id="auto-download-history-list">',
    '<colgroup>',
    '<col span="1" class="list_info">',
    '<col span="1" class="list_status">',
    '</colgroup>',
    '</table>',
    '</div>',
].join('');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    configs.id = [configs.id, 'auto-download-history-dialog-'].join('');

    configs.bodyContent = TEMPLATE_BODY_CONTENT;
    configs.doubleDialog = true;
    configs.contentHeight = configs.contentHeight || '300px';
    configs.buttons = [
        {text: _('dialog_return_menu'), click: dixim.proxy(this.hide, this)},
        {text: _('dialog_history_delete'), click: dixim.proxy(this._clickHistoryDelete, this)},
    ];
    configs.title = _('auto_download_operation_confirm_status');

    superConstructor.prototype._initialize.call(this, configs);
};

prototype._getItemErrorString = function(errorCode) {
    var prefix = 'auto_download_history_error_';
    var description = _([prefix, errorCode].join(''));

    prefix = ['[', prefix].join('');
    if (description.substr(0, prefix.length) === prefix) {
        description = _('auto_download_history_error_unknown');
    }
    return description;
};

prototype.addRow = function(task, item) {
    var date = new Date();
    var year;
    var month;
    var day;
    var hour;
    var minute;
    var time;

    $('#auto-download-history-list').append([
        '<tbody id="', item.itemId, '">',
        '<tr>', '<td>',
        '<span class="list_title_s">', escape(item.title), '</span>',
        '<div class="list_detail txt_s" >',
        '<dl class="list_6em">',
        '<dt>', _('auto_download_history_date'), ':', '</dt>',
        '<dd>', '<p>', '<span class="auto-download-history-date">', '</span>', '</p>', '</dd>',
        '</dl>',
        '<dl class="task-info-', item.taskId, ' list_6em">',
        '<dt>', _('auto_download_history_src'), ':', '</dt>',
        '<dd>', '<span class="auto-download-history-src">', escape(task.srcFriendlyName), '(', task.srcContainerTitle, ')', '</span>', '</dd>',
        '</dl>',
        '</div>',
        '</td>', '<td class="transfer-history-status">',
        '<div class="st_dlok">',
        '<img id="history-img-', item.itemId, '" width="40px" />',
        '<span id="transfer-status">', '</span>',
        '</div>',
        '</td>', '</tr>',
        '</tbody>',
    ].join(''));

    if (item.state === 4) {
        $(['#history-img-', item.itemId].join('')).prop('src', HISTORY_IMAGES['success']);
        $(['#', item.itemId, ' #transfer-status'].join('')).html( _('auto_download_history_success'));
        $(['#', item.itemId, ' #transfer-status'].join('')).addClass('color_green');
    } else {
        if (item.errorCode !== 0) {
            $(['#', item.itemId, ' dl:last-child'].join('')).after([
                '<dl class="list_6em">',
                '<dt>', _('auto_download_history_error_title'), ':',  '</dt>',
                '<dd>', '<p>',
                '<span class="auto-download-history-error">', this._getItemErrorString(item.errorCode), '</span>',
                '</p>', '</dd>',
                '</dl>',
            ].join(''))
        }
        $(['#history-img-', item.itemId].join('')).prop('src', HISTORY_IMAGES['error']);
        $(['#', item.itemId, ' #transfer-status'].join('')).html( _('auto_download_history_error'));
        $(['#', item.itemId, ' #transfer-status'].join('')).addClass('color_red');
    }
    date.setTime(item.lastStateChangedTime * 1000);
    year = date.getFullYear();
    month = parseInt(date.getMonth(), 10)+1;
    day = ('00' + date.getDate()).slice(-2);
    hour = ('00' + date.getHours()).slice(-2);
    minute = ('00' + date.getMinutes()).slice(-2);
    time = year + '/' + month + '/' + day + '  ' + hour + ':' + minute;
    $(['#', item.itemId, ' .auto-download-history-date'].join('')).html(time);
};

prototype.noRow = function() {
    this.disableButton(_('dialog_history_delete'));
    this.error(_('auto_download_history_no_content'));
};

prototype.error = function(text) {
    var selector = $('#auto-download-history-no-content');

    selector.html([
        '<span>', text, '</span>',
    ].join(''));
    selector.addClass('txt_none').addClass('error-text-field');
};

prototype.cleanList = function() {
    $('#auto-download-history-list tbody').remove();
};

prototype._clickHistoryDelete = function() {
    this.dispatchEvent(new Event(HISTORY_DELETE));
};

}());
