/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule cond-dialog-controller
 */

(function() {
var CondDialogController;
var prototype;
var superConstructor;
var CondDialogView = dixim.autoDownload.CondDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var Enum = dixim.ctt.util.Enum;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var _ = dixim.ctt.ResourceBundle.get;

/**
 * @class CondDialogController
 * @constructor
 * @param {Object} configs
 */
CondDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(CondDialogController, EventTarget);
dixim.autoDownload.CondDialogController = CondDialogController;
prototype = CondDialogController.prototype;
superConstructor = CondDialogController.superConstructor;

var NO_COND = 'noCond';
var SETTING = 'setting';
var CLOSE = 'close';

var COND = Enum.fetch('COND');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'cond-dialog'].join('');

    this._condId = configs.condId;
    this._conds = configs.conds || [];

    this._view = new CondDialogView(configs);
    this._view.addEventListener(SETTING, this._clickSetting, this);
    this._view.addEventListener(NO_COND, this._clickNoCond, this);
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._setConds();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._setConds = function() {
    switch(this._condId) {
        case COND.TITLE:
        case COND.CHANNEL:
        case COND.DESCRIPTION:
        case COND.DATE:
            this._view.setCondsDefault(this._conds);
            break;
        case COND.GENRE:
            this._view.setCondsGenre(this._conds);
            break;
        case COND.DAY:
            this._view.setCondsDay(this._conds);
            break;
        case COND.DURATION:
            this._view.setCondsDuration(this._conds);
            break;
        case COND.START_TIME:
        case COND.END_TIME:
            this._view.setCondsRecording(this._conds);
            break;
        case COND.BROADCAST:
            this._view.setCondsBroadcast(this._conds);
            break;
        case COND.COPY_COUNT:
            this._view.setCondsCopyCount(this._conds);
            break;
    }
};

prototype._clickSetting = function(e) {
    var ret = {
        isError: false,
        name: e.args[0].name,
        conds: e.args,
    };
    var callback = dixim.proxy(function() {
        this.dispatchEvent(new Event(SETTING, ret));
        this.stop();
    }, this);

    for (var i = 0; i < ret.conds.length; i += 1) {
        if (ret.conds[i].value === 'error') {
            ret.isError = true;
            ret.conds.splice(i, 1);
            i -= 1;
        }
    }
    if (ret.isError) {
        this._errorDialog(_('auto_download_cond_error_description'), dixim.proxy(callback, this));
    } else {
        callback();
    }
};

prototype._clickNoCond = function(e) {
    this.dispatchEvent(new Event(NO_COND, e.args));
    this.stop();
};

prototype._errorDialog = function(text, closeCallback) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _(['auto_download_filter_edit_condition_', this._condId].join(''));
    query.description = text;
    query.doubleDialog = true;
    dialog = new ErrorModalDialogView(query);
    if (typeof closeCallback !== 'undefined') {
        dialog.addEventListener(CLOSE, dixim.proxy(closeCallback, this), this);
    }
    dialog.show();
};

}());
