/**
 * @module dixim
 * @submodule array
 * @class array
 */
dixim.namespace('dixim.array');

/**
 * Returns the index of the first item in the array
 * that equals to the object specified.
 *
 * @method findIndex
 * @static
 * @param {Array} array an Array to be scanned.
 * @param {Object} obj a target object.
 * @return {Number} the index of the <em>obj</em> if it's found.
 *  Otherwise the length of the <em>array</em>.
 */
dixim.array.findIndex = function(array, obj) {
    var len = array.length;
    var i;

    for (i = 0; i < len; i += 1) {
        if (array[i] === obj) return i;
    }
    return len;
};

/**
 * Removes the first item in the array that equals to the object specified.
 *
 * @method remove
 * @static
 * @param {Array} array an Array to be manipulated.
 * @param {Object} obj a target object.
 * @return {Object} the object removed from the <em>array</em>, null otherwise.
 */
dixim.array.remove = function(array, obj) {
    var len = array.length;
    var i;

    if (!len) return null;
    if ((i = dixim.array.findIndex(array, obj)) === len) return null;
    return Array.prototype.splice.call(array, i, 1)[0];
};

/**
 * Iterates the array with the callback specified.
 *
 * When calling the function, two arguments are passed.
 * First argument is an index of the array, second is element value.
 * If the function returns false, the iteration is stopped.
 *
 * @static
 * @param {Array} array an Array to be iterated.
 * @param {Function} callback a callback function.
 * @return {Array} an Array passed to this method.
 * @example
 *     dixim.array.each([1, 2, 3, 4, 5, 6, 7, 8, 9], function(i, v) {
 *       if (v > 5) return false;
 *       if (v % 2) return;
 *       console.log(v);
 *     }); // prints 2, 4
 */
dixim.array.each = function(array, callback) {
    var len = array.length;
    var i;

    for (i = 0; i < len; i += 1) {
        if (callback.apply(array, [i, array[i]]) === false) break;
    }

    return array;
};
