/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule permission-default-setting-controller
 */

(function() {
var PermissionDefaultSettingController;
var prototype;
var superConstructor;
var PermissionDefaultSettingView = dixim.server.PermissionDefaultSettingView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientServer = dixim.common.ApiClientServer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var BlockUi = dixim.common.BlockUi;

/**
 * @class PermissionDefaultSettingController
 * @constructor
 * @param {Object} configs
 */
PermissionDefaultSettingController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(PermissionDefaultSettingController, EventTarget);
dixim.server.PermissionDefaultSettingController = PermissionDefaultSettingController;
prototype = PermissionDefaultSettingController.prototype;
superConstructor = PermissionDefaultSettingController.superConstructor;

var PREV = 'prev';
var APPLY = 'apply';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'permission-default-setting'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('getDefaultPermission', this._successGetDefaultPermission, this);
    this._model.addEventListener('getDefaultPermissionError', this._errorGetDefaultPermission, this);
    this._model.addEventListener('setDefaultPermission', this._successSetDefaultPermission, this);
    this._model.addEventListener('setDefaultPermissionError', this._errorSetDefaultPermission, this);

    this._view = new PermissionDefaultSettingView(configs);
    this._view.addEventListener(PREV, this._clickPrev, this);
    this._view.addEventListener(APPLY, this._clickApply, this);

    this._blockUi = new BlockUi();
    this._parentContext = configs.parentContext;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._getDefaultPermission();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._getDefaultPermission = function() {
    this._blockUi.block();
    this._model.getDefaultPermission();
};

prototype._successGetDefaultPermission = function(e) {
    var defaultPermission = e.args.DefaultPermission;

    if (defaultPermission === 'Allow') {
        this._view.setSetting(true);
    } else if (defaultPermission === 'Deny') {
        this._view.setSetting(false);
    } else {
        this._view.setSetting();
    }
    this._blockUi.unblock();
};

prototype._errorGetDefaultPermission = function() {
    this._errorDialog(_('server_default_permission_fetch_error'));
    this._view.setSetting();
};

prototype._successSetDefaultPermission = function() {
    this._blockUi.unblock();
    this._getDefaultPermission();
};

prototype._errorSetDefaultPermission = function() {
    this._blockUi.unblock();
    this._errorDialog(_('server_default_permission_set_error'));
};

prototype._errorDialog = function(text) {
    var dialog = null;
    var query = {};

    query.id = this._id;
    query.title = _('server_permission_menu_default');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._clickApply = function(e) {
    var isAllow = e.args.isAllow;
    var query = {};

    this._blockUi.block();

    query.DefaultPermission = (isAllow) ? 'Allow' : 'Deny';
    this._model.setDefaultPermission(query);
};

}());
