/**
 * @module ctt
 * @submodule transfer-status-listing-view
 */
(function() {
var ItemListingView = dixim.ctt.ItemListingView;
var $ = dixim.ctt.jQuery;
var escapeHtml = dixim.xml.escape;

var TransferStatusListingView;
var superConstructor;
var prototype;

/**
 * @class TransferStatusListingView
 * @extends ItemListingView
 * @constructor
 * @param {Object} configs
 */
TransferStatusListingView = function(configs) {
  ItemListingView.call(this, configs);
};
dixim.inherit(TransferStatusListingView, ItemListingView);
dixim.ctt.TransferStatusListingView = TransferStatusListingView;
prototype = TransferStatusListingView.prototype;
superConstructor = TransferStatusListingView.superConstructor;

prototype._initialize = function(configs) {
  var cfg = {
    className: 'ctt-transfer-status-listing-view',
    elementIdPrefix: 'ctt-transfer-status-listing-view-'
  };

  cfg = dixim.extend(cfg, configs);
  superConstructor.prototype._initialize.call(this, cfg);
};

var TEMPLATE_ROW = ['<li id="${id}" class="ctt-item-listing-view-row">',
                    '<div class="ctt-item-listing-view-row-title">${title}</div>',
                    '<div class="ctt-transfer-status-listing-view-row-progress">${progress}</div>',
                    '</li>'].join('');

var TEMPLATE_ROW_PROGRESS = '<div class="ctt-transfer-status-listing-view-row-progress-gauge"></div>';

/**
 * @method _createRowElementHtml
 * @param {Object} record
 * @param {Number} index
 * @protected
 */
prototype._createRowElementHtml = function(record, index) {
  var id;
  var html;
  var title;

  if (!!!(id = this._makeElementId(record))) {
    return null;
  }

  title = this._getRecordProperty(record, 'title') || '';
  html = TEMPLATE_ROW.
         replace('${id}', id).
         replace('${title}', escapeHtml(title)).
         replace('${progress}', TEMPLATE_ROW_PROGRESS);

  return html;
};

prototype._createRowElementNode = function(record, index, html) {
  var node;

  node = superConstructor.prototype._createRowElementNode.apply(this, arguments);
  this._updateProgressbar(node, record);
  return node;
};

prototype._updateProgressbar = function(node, record) {
  var total;
  var transferred;
  var ratio;
  var status;

  total = record.total;
  transferred = record.transferred;
  status = record.status;

  if (status === 'transferring' || status === 'waiting') {
    node.addClass(['ctt-transfer-status-listing-view-row-', status].join(''));

    if (!!!total || typeof transferred === 'undefined') {
      return node;
    }
    ratio = transferred ? Math.floor((transferred / total) * 100) : 0;
  } else {
    if (status === 'done' || status === 'error' || status === 'cancelled') {
      node.addClass(['ctt-transfer-status-listing-view-row-', status].join(''));
    } else {
      node.addClass('ctt-transfer-status-listing-view-row-warning');
    }

    ratio = 100;
  }

  $('.ctt-transfer-status-listing-view-row-progress-gauge', node).
    progressbar({value: ratio});
};

prototype.updateRecords = function(records) {
  var i;
  var len = records.length;
  var record;
  var newRecord;
  var newRecords = [];
  var node;

  for (i = 0; i < len; i += 1) {
    newRecord = records[i];
    record = this._getRecord(newRecord.id);
    if (!!!record) {
      newRecords.push(newRecord);
      continue;
    }
    dixim.extend(record, newRecord);
    node = this._getRowElement(record);
    this._updateProgressbar(node, record);
  }

  if (newRecords.length) {
    this.addRows(newRecords);
  }
};

}());
