/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule compression-setting-dialog-view
 */

(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var ConpressionSettingDialogView;
var prototype;
var superConstructor;

/**
 * @class ConpressionSettingDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
ConpressionSettingDialogView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(ConpressionSettingDialogView, ModalDialogView);
dixim.ctt.ConpressionSettingDialogView = ConpressionSettingDialogView;
prototype = ConpressionSettingDialogView.prototype;
superConstructor = ConpressionSettingDialogView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div class="ctt-compression-setting-view-wrapper" />';

var TEMPLATE_BODY_CONTENT = '<div class="ctt-compression-setting-view"></div>';

/**
 * Triggered when the cancel button was clicked.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the setting button was clicked.
 *
 * @event setting
 */
var SETTING = 'setting';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;
  var bodyContent;

  configs  = configs || {};

  buttons = [
    {
      'text': _('ctt_compression_setting_button_setting'),
      click: $.proxy(this._didClickSetting, this)
    },
    {
      'text': _('ctt_dialog_button_label_cancel'),
      click: $.proxy(this._didClickCancel, this),
      align: 'left'
    }
  ];

  buttonsError = [
    {
      'text': _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    }
  ];

  bodyContent = TEMPLATE_BODY_CONTENT;

  configs.title = configs.title || _('ctt_compression_setting_title');
  configs.contentBox = configs.contentBox || TEMPLATE_CONTENT_BOX;
  configs.bodyContent = configs.bodyContent || bodyContent;
  configs.buttons = buttons;
  configs.width = 480;
  configs.height = 'auto';

  this._descriptionNode = null;
  this._model = new ContentsTransferApiClient();
  this.fetch_setting = {};

  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didRenderDialog = function(e) {
  this._didCreateDialog();
  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didCreateDialog = function(e) {
  $('.ctt-compression-setting-view').html(
    [
      '<div class="ctt-compression-setting-view-availability-description"></div>',
      '<div class="ctt-compression-setting-view-availability-select"></div>',
      '<div class="ctt-compression-setting-view-preset-description"></div>',
      '<div class="ctt-compression-setting-view-preset-list"></div>',
      '</div>'
     ].join('')
  );
  $('.ctt-compression-setting-view-availability-description').html(
    [
      _('ctt_compression_setting_availability')
    ].join('')
  );
  $('.ctt-compression-setting-view-availability-select').html(
    [
     '<input type="radio" name="availability" class="yes">',
     '<span>',_('ctt_compression_setting_availability_yes'),'</span>',
     '<input type="radio" name="availability" class="no">',
     '<span>',_('ctt_compression_setting_availability_no'),'</span>'
     ].join('')
    );
  $('.ctt-compression-setting-view-preset-description').html(
    [
      _('ctt_compression_setting_preset')
    ].join('')
  );
  $('.ctt-compression-setting-view-preset-list').html(
    [
      '<table>',
      '</table>'
    ].join('')
  );
  this._model.addEventListener('fetchPreset', this._didCreatePresetList, this);
  this._model.fetchPreset();
  this._model.addEventListener('fetchAvailabilityCompression', this._didSetAvailability, this);
  this._model.fetchAvailabilityCompression();
};

prototype._didCreatePresetList = function(e) {
  var presets = e.args.data.presets;
  var presets_length = presets.length;

  for (var i = 0; i < presets_length; i += 1) {
    this._didAddPresetList(presets[i]);
  }
};

prototype._didAddPresetList = function(preset) {
  $('.ctt-compression-setting-view-preset-list table').append(
    [
     '<tr><td>',
     '<input type="radio" name="presets" id="', preset.id,'">',
     '</td><td>',
     '<span>',preset.name,'</span>',
     '</td></tr>'
    ].join('')
  );
  if (preset.selected) {
    this.fetch_setting.preset_id = preset.id;
    $(['#',preset.id].join('')).prop('checked', true);
  }
};

prototype._didSetAvailability = function(e) {
  var availability = e.args.data.availability;

  this.fetch_setting.availability = availability;
  if (availability) {
    $('.ctt-compression-setting-view-availability-select .yes').prop('checked', true);
  } else {
    $('.ctt-compression-setting-view-availability-select .no').prop('checked', true);
  }
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickCancel = function() {
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

prototype._didClickClose = function() {
  this._didCancel();
};

prototype._didClickSetting = function() {
  var data = {};
  var after_setting = {
    availability: $('.ctt-compression-setting-view-availability-select .yes').is(':checked'),
    preset_id: parseInt($('.ctt-compression-setting-view-preset-list input:checked').prop('id'), 10)
  };

  if (after_setting.availability !== this.fetch_setting.availability) {
    data.availability = after_setting.availability;
  }
  if (after_setting.preset_id !== this.fetch_setting.preset_id) {
    data.preset_id = after_setting.preset_id;
  }
  this.dispatchEvent(new Event(SETTING, data));
};

}());
