/**
 * @module ctt
 * @submodule schedule-entry-edit--wizard-page-view
 */
(function() {
var Event = dixim.event.Event;
var WizardPageView = dixim.ctt.WizardPageView;
var TickerView = dixim.ctt.TickerView;
var Form = dixim.ctt.util.Form;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var ScheduleEntryEditWizardPageView;
var prototype;
var superConstructor;

/**
 * @class ConfirmationWizardPageView
 * @extends WizardPageView
 * @constructor
 * @param {Object} configs
 */
ScheduleEntryEditWizardPageView = function(configs) {
  WizardPageView.call(this, configs);
};
dixim.inherit(ScheduleEntryEditWizardPageView, WizardPageView);
dixim.ctt.ScheduleEntryEditWizardPageView = ScheduleEntryEditWizardPageView;
prototype = ScheduleEntryEditWizardPageView.prototype;
superConstructor = ScheduleEntryEditWizardPageView.superConstructor;

var TEMPLATE_SCHEDULE = ['<span class="ctt-schedule-entry-edit-wizard-page-view-form-schedule-date">${date}</span>',
                         '<span class="ctt-schedule-entry-edit-wizard-page-view-form-schedule-time">',
                         Form.createTimeSelector(),
                         '</span>'].join('');

var TEMPLATE = ['<div class="ctt-schedule-entry-edit-wizard-page-view">',
                  '<div class="ctt-schedule-entry-edit-wizard-page-view-message-wrapper">',
                    '<div class="ctt-schedule-entry-edit-wizard-page-view-description">${description}</div>',
                    '<div class="ctt-schedule-entry-edit-wizard-page-view-error"></div>',
                  '</div>',
                  '<div class="ctt-schedule-entry-edit-wizard-page-view-form-wrapper">',
                    '<form class="ctt-schedule-entry-edit-wizard-page-view-form"><table>',
                      '<tr class="ctt-schedule-entry-edit-wizard-page-view-form-row-current-schedule">',
                        '<td>${currentSchedule}</td>',
                        '<td></td></tr>',
                      '<tr class="ctt-schedule-entry-edit-wizard-page-view-form-row-new-schedule">',
                        '<td>${newSchedule}</td><td>',
                          TEMPLATE_SCHEDULE,
                        '</td></tr>',
                    '</table></form>',
                  '</div>',
                '</div>'
                ].join('');

var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

/**
 * @event render
 */
var RENDER = 'render';

/**
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * @event save
 */
var SAVE = 'save';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;
  var cfg;
  var body;

  body = TEMPLATE.
         replace('${date}', Form.createDateSelector()).
         replace('${description}', _('ctt_schedule_entry_edit_wizard_page_view_description')).
         replace('${currentSchedule}', _('ctt_schedule_entry_edit_wizard_page_view_form_current_schedule_row_label')).
         replace('${newSchedule}', _('ctt_schedule_entry_edit_wizard_page_view_form_new_schedule_row_label'));

  cfg = {
    title: _('ctt_schedule_entry_edit_wizard_page_view_title'),
    bodyContent: body
  };

  cfg = dixim.extend(cfg, configs);
  buttons = [
    {
      text: _('ctt_schedule_entry_edit_wizard_page_view_button_label_save'),
      click: $.proxy(this._didClickSave, this)
    },
    {
      text: _('ctt_dialog_button_label_cancel'),
      click: $.proxy(this._didClickCancel, this),
      align: 'left'
    }
  ];

  cfg.buttons = buttons;
  this._form = null;
  this._ticker = null;
  superConstructor.prototype._initialize.call(this, cfg);
};

prototype._didRenderDialog = function(e) {
  var v = this._view;
  var form;
  var date;
  var min;
  var hour;
  var msec;

  date = Form.getNextDateRange();
  min = date.getMinutes();
  hour = date.getHours();
  msec = new Date(date.getFullYear(), date.getMonth(), date.getDate()).getTime();

  form = $('.ctt-schedule-entry-edit-wizard-page-view-form', v);
  $('select[name="date"] option[value="' + msec + '"]', form).attr({'selected': true});
  $('select[name="hour"] option[value="' + hour + '"]', form).attr({'selected': true});
  $('select[name="min"] option[value="' + min + '"]', form).attr({'selected': true});
  this._form = form;

  this._ticker = new TickerView({
    containerNode: $('.ctt-schedule-entry-edit-wizard-page-view-error', v),
    type: 'error'
  });
  this._loading = $(TEMPLATE_LOADING);

  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickSave = function(e) {
  this.dispatchEvent(new Event(SAVE));
};

prototype._didClickCancel = function(e) {
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

prototype.destroy = function() {
  if (!!this._ticker) {
    this._ticker.destroy();
  }
  superConstructor.prototype.destroy.call(this);
};

/**
 * @method getSelectedDate
 * @return {Date}
 */
prototype.getSelectedDate = function() {
  var date;
  var hour;
  var min;

  date = parseInt($('select[name="date"] option:selected', this._form).val(), 10);
  hour = parseInt($('select[name="hour"] option:selected', this._form).val(), 10);
  min = parseInt($('select[name="min"] option:selected', this._form).val(), 10);

  date = new Date(date);
  date.setHours(hour, min, 0, 0);
  return date;
};

/**
 * @method setTargetSchedule
 * @param {Object} schedule schedule data to be modified.
 */
prototype.setTargetSchedule = function(schedule) {
  var re = /^([0-9]{4})-([0-9]{2})-([0-9]{2})T([0-9]{2}):([0-9]{2}):([0-9]{2})(\.([0-9]{3}))?$/;
  var slice = Array.prototype.slice;
  var el;
  var ret;
  var text;

  ret = re.exec(schedule.startTime);
  if (!!!ret) return;

  el = $('.ctt-schedule-entry-edit-wizard-page-view-form-row-current-schedule', this._form);
  text = slice.call(ret, 1, 4).join('/');
  text = [text, slice.call(ret, 4, 6).join(':')].join(' ');
  $('> td + td', el).text(text);
};

prototype.showError = function(error) {
  this._ticker.setText(error);
  this._ticker.show();
};

prototype.showActivityIndicator = function() {
  var buttonSet;

  buttonSet = $('.ui-dialog-buttonset', this._view.parent());
  buttonSet.append(this._loading);
  this._ticker.hide(false);
  this.disableButtons();
  this.hideCloseButton();
};

prototype.hideActivityIndicator = function() {
  this._loading.detach();
  this.enableButtons();
  this.showCloseButton();
};

}());
