/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule progress-dialog-view
 */

(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var TickerView = dixim.ctt.TickerView;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var ProgressDialogView;
var prototype;
var superConstructor;

/**
 * @class ProgressDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
ProgressDialogView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(ProgressDialogView, ModalDialogView);
dixim.ctt.ad.ProgressDialogView = ProgressDialogView;
prototype = ProgressDialogView.prototype;
superConstructor = ProgressDialogView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div class="ctt-deletion-view-wrapper" />';

var TEMPLATE_BODY_CONTENT = ['<div class="ctt-deletion-view">',
                               '<div class="ctt-deletion-view-description"></div>',
                               '<div class="ctt-deletion-view-error"></div>',
                               '<div class="ctt-deletion-view-progress">',
                               '</div>',
                             '</div>'].join('');

var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

/**
 * Triggered when the close button was clicked.
 *
 * @event delete
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;
  var buttonsError;
  var bodyContent;

  configs  = configs || {};

  buttons = [
    {
      'text': _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    }
  ];

  buttonsError = [
    {
      'text': _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    }
  ];

  bodyContent = TEMPLATE_BODY_CONTENT;

  configs.title = configs.title || _('ctt_ad_progress_dialog_add_task_title');
  configs.contentBox = configs.contentBox || TEMPLATE_CONTENT_BOX;
  configs.bodyContent = configs.bodyContent || bodyContent;
  configs.buttons = buttons;
  configs.width = 480;
  configs.height = 'auto';
  configs.maxHeight = 340;

  this._descriptionNode = null;
  this._errorNode = null;
  this._progressNode = null;
  this._ticker = null;
  this._targetItems = configs.targetItems;
  this._loadingIcon = null;
  this._inProgress = false;
  this._errorButtons = buttonsError;
  this._descriptionStringKey = configs.description || 'ctt_ad_progress_dialog_add_task_end';
  this._inProgressStringKey = configs.in_progress || 'ctt_ad_progress_dialog_add_task_description';

  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didRenderDialog = function(e) {
  var v = this._view;

  this._descriptionNode = $('.ctt-deletion-view-description', v).text(_(this._descriptionStringKey));
  this._progressNode = $('.ctt-deletion-view-progress', v);
  this._ticker = new TickerView({
    containerNode: $('.ctt-deletion-view-error', v),
    type: 'error'
  });
  this._loadingIcon = $(TEMPLATE_LOADING);
  this._descriptionNode.append(this._loadingIcon);

  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickClose = function() {
  this.dispatchEvent(new Event(CLOSE));
};

prototype.inProgress = function() {
  this._descriptionNode.empty();
  this._progressNode.text(_(this._inProgressStringKey));
};

prototype.finish = function() {
  this._progressNode.empty();
  this._descriptionNode.text(_(this._descriptionStringKey));
};

/**
 * @method showError
 * @param {String} error
 */
prototype.showError = function(error) {
  this._showError(error);
};

prototype._showError = function(error) {
  this._ticker.setText(error);
  this._ticker.show();
  this.setButtons(this._errorButtons);
};

}());
