/**
 * @module ctt
 * @submodule dms-listing-wizard-page-controller
 */
(function() {
var Event = dixim.event.Event;
var WizardPageController = dixim.ctt.WizardPageController;
var DmsListingController = dixim.ctt.DmsListingController;
var DestinationStorageListingWizardPageController = dixim.ctt.DestinationStorageListingWizardPageController;
var ConfirmationWizardPageController = dixim.ctt.ConfirmationWizardPageController;
var ScheduledRecordingWizardPageController = dixim.ctt.ScheduledRecordingWizardPageController;
var DmsListingWizardPageView = dixim.ctt.DmsListingWizardPageView;
var _ = dixim.ctt.ResourceBundle.get;

var DmsListingWizardPageController;
var prototype;
var superConstructor;
var makeFilter;

/**
 * @class DmsListingWizardPageController
 * @extends WizardPageController
 * @uses DmsListingController
 * @uses DmsListingWizardPageView
 * @uses ContentsTransferApiClient
 * @uses Event
 * @constructor
 * @param {Object} configs
 */
DmsListingWizardPageController = function(configs) {
  WizardPageController.call(this, configs);
};
dixim.inherit(DmsListingWizardPageController, WizardPageController);
dixim.ctt.DmsListingWizardPageController = DmsListingWizardPageController;
prototype = DmsListingWizardPageController.prototype;
superConstructor = DmsListingWizardPageController.superConstructor;

makeFilter = function(items) {
  var len = items.length;
  var ret = [];
  var i;
  var item;

  for (i = 0; i < len; i += 1) {
    item = items[i];
    if (!!!item.profile) continue;
    ret.push(item.profile);
  }
  return ret.join(',');
};


/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  superConstructor.prototype._initialize.call(this, configs);

  this._view = new DmsListingWizardPageView();
  this._dmsListingController = null;
  this._model = null;
  this._session = null;
  this._enableWol = true;
};

prototype.start = function() {
  var v = this._view;
  var c;

  v.addEventListener('render', this._didRenderView, this);
  v.addEventListener('cancel', this._didCancel, this);
  v.addEventListener('next', this._didClickNext, this);

  if (!!(c = this._dmsListingController)) {
    c.start();
  }
  superConstructor.prototype.start.call(this);
};

prototype._didRenderView = function(e) {
  var dlc;
  var model;
  var targetItems;
  var filter;
  var wol;
  var w;
  var v = this._view;

  if (!!!(w = this._wizard)) {
    throw new Error('not pushed into wizard stack.');
  }

  wol = w.getRegistryEntry('enableWol');
  model = w.getRegistryEntry('model');
  targetItems = w.getRegistryEntry('targetItems');
  filter = makeFilter(targetItems);

  dlc = new DmsListingController({
    containerNode: v.getBodyContent(),
    model: model,
    filter: filter
  });
  dlc.addEventListener('changeSelection', this._didChangeSelection, this);
  dlc.addEventListener('updateList', this._didUpdateList, this);
  dlc.addEventListener('updateListError', this._didFailUpdateList, this);
  dlc.start();
  this._dmsListingController = dlc;

  if (wol) {
    model.addEventListener('removeDms', this._didFinishRemoveDmsRequest, this);
    model.addEventListener('removeDmsError', this._didFinishRemoveDmsRequest, this);
    v.addEventListener('delete', this._didClickDelete, this);
  } else {
    v.hideDeleteButton();
  }
  this._enableWol = wol;
  this._model = model;
};

prototype._didChangeSelection = function(e) {
  var dms;

  dms = this._dmsListingController.getSelectedDms();
  if (!!dms) {
    this._view.enableNextButton();
    if (this._enableWol) {
      if (typeof dms.alive !== 'undefined' && !dms.alive) {
        this._view.enableDeleteButton();
      } else {
        this._view.disableDeleteButton();
      }
    }
  } else {
    if (this._enableWol) {
      this._view.disableDeleteButton();
    }
    this._view.disableNextButton();
  }
};

prototype._didUpdateList = function(e) {
  var w = this._wizard;
  var dlc = this._dmsListingController;
  var dms;
  var num;
  var el;

  if (!(num = dlc.getNumberOfDmss())) {
    this._view.showError(_('ctt_dms_listing_wizard_page_view_error_no_dms_available'));
    return;
  }
  if (!!!w) return;

  if (num === 1) {
    dlc.selectFirstDms();
  } else if (!!(dms = w.getRegistryEntry('dms'))) {
    dlc.selectDms(dms);
    el = dlc.getSelectedDmsElement();
    if (!!el) {
      this._view.scrollToElement(el);
    }
  }
};

prototype._didFailUpdateList = function(e) {
  this._view.showError(_('ctt_dms_listing_wizard_page_view_error'));
};

prototype._didClickNext = function(e) {
  var w = this._wizard;
  var dms;
  var nc;
  var schedule;

  dms = this._dmsListingController.getSelectedDms();
  if (!!!dms) {
    return;
  }
  w.setRegistryEntry('dms', dms);

  schedule = w.getRegistryEntry('enableScheduledTransferring');
  if (dms.hdlnkdoc.match(/REC-.+/)) {
    nc = new DestinationStorageListingWizardPageController();
  } else if (dms.moveRes && schedule) {
    nc = new ScheduledRecordingWizardPageController();
  } else {
    nc = new ConfirmationWizardPageController();
  }

  w.pushViewController(nc);
};

prototype._didClickDelete = function(e) {
  var dms;

  if (!!this._session) return;

  this._view.showActivityIndicator();
  dms = this._dmsListingController.getSelectedDms();
  this._session = this._model.removeDms({udn: dms.udn});
};

prototype._didFinishRemoveDmsRequest = function(e) {
  var session = e.args.session;

  if (this._session !== session) return;

  this._view.hideActivityIndicator();
  this._session = null;
  this._dmsListingController.reload();
};

prototype.stop = function() {
  if (!!this._dmsListingController) {
    this._dmsListingController.stop();
  }
  superConstructor.prototype.stop.call(this);
};

prototype.destroy = function() {
  var m;

  if (!!this._dmsListingController) {
    this._dmsListingController.destroy();
  }
  if (!!(m = this._model) && this._enableWol) {
    m.removeEventListener('removeDms', this._didRemoveDms, this);
    m.removeEventListener('removeDmsError', this._didFailRemoveDms, this);
  }
  superConstructor.prototype.destroy.call(this);
};

}());
