/**
 * @module ctt
 * @submodule locking-view
 */
(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var TickerView = dixim.ctt.TickerView;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var LockingView;
var prototype;
var superConstructor;

/**
 * @class LockingView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
LockingView = function(configs) {
  ModalDialogView.call(this, configs);
  this._initialize(configs);
};
dixim.inherit(LockingView, ModalDialogView);
dixim.ctt.LockingView = LockingView;
prototype = LockingView.prototype;
superConstructor = LockingView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div class="ctt-locking-view-wrapper" />';

var TEMPLATE_BODY_CONTENT = ['<div class="ctt-locking-view">',
                               '<div class="ctt-locking-view-description"></div>',
                               '<div class="ctt-locking-view-error"></div>',
                               '<div class="ctt-locking-view-progress">',
                               '</div>',
                             '</div>'].join('');

var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

/**
 * Triggered when the cancel button was clicked.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the lock button was clicked.
 *
 * @event lock
 */
var LOCK = 'lock';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;
  var buttonsError;
  var bodyContent;

  configs  = configs || {};

  buttons = [
    {
      'text': _('ctt_locking_view_button_label_lock'),
      click: $.proxy(this._didClickLock, this)
    },
    {
      'text': _('ctt_dialog_button_label_cancel'),
      click: $.proxy(this._didClickCancel, this),
      align: 'left'
    }
  ];

  buttonsError = [
    {
      'text': _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    }
  ];

  bodyContent = TEMPLATE_BODY_CONTENT;

  configs.title = _('ctt_locking_view_title');
  configs.contentBox = configs.contentBox || TEMPLATE_CONTENT_BOX;
  configs.bodyContent = configs.bodyContent || bodyContent;
  configs.buttons = buttons;
  configs.width = 480;
  configs.height = 'auto';
  configs.maxHeight = 340;

  this._descriptionNode = null;
  this._errorNode = null;
  this._progressNode = null;
  this._ticker = null;
  this._targetItems = configs.targetItems;
  this._loadingIcon = null;
  this._inProgress = false;
  this._errorButtons = buttonsError;

  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didRenderDialog = function(e) {
  var v = this._view;

  this._descriptionNode = $('.ctt-locking-view-description', v).
                            text(_('ctt_locking_view_description',
                                 this._targetItems.length));
  this._progressNode = $('.ctt-locking-view-progress', v);
  this._ticker = new TickerView({
    containerNode: $('.ctt-locking-view-error', v),
    type: 'error'
  });
  this._loadingIcon = $(TEMPLATE_LOADING);

  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickCancel = function(e) {
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

prototype._didClickClose = function() {
  this._didCancel();
};

prototype._didClickLock = function(e) {
  this.dispatchEvent(new Event(LOCK));
};

/**
 * @method showError
 * @param {String} error
 */
prototype.showError = function(error) {
  this._showError(error);
};

prototype._showError = function(error) {
  this._ticker.setText(error);
  this._ticker.show();
  this.setButtons(this._errorButtons);
};

prototype.updateProgress = function(currentItem, index, total) {
  this._progressNode.
        text(_('ctt_locking_view_progress',
               currentItem.title, index + 1, total));
};

prototype.showActivityIndicator = function() {
  if (this._inProgress) return;

  this._inProgress = true;
  this.disableButtons();
  this.hideCloseButton();
  this._descriptionNode.text(_('ctt_locking_view_description_in_progress'));
  this._descriptionNode.append(this._loadingIcon);
};

prototype.hideActivityIndicator = function() {
  if (!this._inProgress) return;

  this._inProgress = false;
  this._loadingIcon.detach();
  this.enableButtons();
  this.showCloseButton();
};

}());
