/**
 * @module ctt
 * @submodule transfer-history-listing-view
 */
(function() {
var ItemListingView = dixim.ctt.ItemListingView;
var Formatter = dixim.ctt.util.Formatter;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;
var escapeHtml = dixim.xml.escape;

var TransferHistoryListingView;
var superConstructor;
var prototype;

var TEMPLATE_SCROLLER = ['<div class="ctt-scroller ctt-scroller-with-header">',
                           '<div class="ctt-scroller-header ctt-transfer-history-listing-view-scroller-header">',
                             '<table class="ctt-transfer-history-listing-view-header">',
                               '<tr>',
                                 '<th class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-date">${date}</th>',
                                 '<th class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-status">${status}</th>',
                                 '<th class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-title">${title}</th>',
                                 '<th class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-destination">${destination}</th>',
                               '</tr>',
                             '</table>',
                           '</div>',
                           '<div class="ctt-scroller-content"></div>',
                         '</div>'].join('');

var TEMPLATE_CONTAINER = '<table class="ctt-transfer-history-listing-view"/>';

var TEMPLATE_CELL_BOUNDS_HARNESS = '<div class="ctt-table-cell-bounds-harness">${cellContent}</div>';

var TEMPLATE_CELL_DESTINATION = '${storage} (${server})';

var TEMPLATE_ROW = ['<tr id="${id}" class="ctt-transfer-history-listing-view-row">',
                      '<td class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-date">${date}</td>',
                      '<td class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-status">${status}</td>',
                      '<td class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-title">${title}</td>',
                      '<td class="ctt-transfer-history-listing-view-cell ctt-transfer-history-listing-view-cell-destination">${destination}</td>',
                    '</tr>'].join('');

/**
 * @class TransferHistoryListingView
 * @extends ItemListingView
 * @constructor
 * @param {Object} configs
 */
TransferHistoryListingView = function(configs) {
  ItemListingView.call(this, configs);
};
dixim.inherit(TransferHistoryListingView, ItemListingView);
dixim.ctt.TransferHistoryListingView = TransferHistoryListingView;
prototype = TransferHistoryListingView.prototype;
superConstructor = TransferHistoryListingView.superConstructor;

prototype._initialize = function(configs) {
  var cfg = {
    className: 'ctt-transfer-history-listing-view',
    elementIdPrefix: 'ctt-transfer-history-listing-view-'
  };
  var schema = {
      id: 'date'
  };

  cfg = dixim.extend(cfg, configs);
  cfg.schema = schema;
  cfg.containerTemplate = TEMPLATE_CONTAINER;
  cfg.rowTemplate = TEMPLATE_ROW;
  superConstructor.prototype._initialize.call(this, cfg);

  this._scroller = null;
};

var LogStatus = {
  '0': _('ctt_transfer_history_listing_view_log_message_success'),
  '1': _('ctt_transfer_history_listing_view_log_message_cancelled'),
  'SS401': _('ctt_transfer_history_listing_view_log_message_SS401'),
  'SS402': _('ctt_transfer_history_listing_view_log_message_SS402'),
  'SS501': _('ctt_transfer_history_listing_view_log_message_SS501'),
  'SS600': _('ctt_transfer_history_listing_view_log_message_SS600'),
  'SS603': _('ctt_transfer_history_listing_view_log_message_SS603'),
  'AS801': _('ctt_transfer_history_listing_view_log_message_AS801'),
  'AS802': _('ctt_transfer_history_listing_view_log_message_AS802'),
  'AS803': _('ctt_transfer_history_listing_view_log_message_AS803'),
  'AS804': _('ctt_transfer_history_listing_view_log_message_AS804'),
  'AS805': _('ctt_transfer_history_listing_view_log_message_AS805'),
  'AS809': _('ctt_transfer_history_listing_view_log_message_AS809'),
  'AS820': _('ctt_transfer_history_listing_view_log_message_AS820'),
  'AS830': _('ctt_transfer_history_listing_view_log_message_AS830'),
  'AS831': _('ctt_transfer_history_listing_view_log_message_AS831'),
  'AS836': _('ctt_transfer_history_listing_view_log_message_AS836'),
  'AS841': _('ctt_transfer_history_listing_view_log_message_AS841'),
  'AS845': _('ctt_transfer_history_listing_view_log_message_AS845'),
  'AS853': _('ctt_transfer_history_listing_view_log_message_AS853'),
  'AS860': _('ctt_transfer_history_listing_view_log_message_AS860'),
  'default': _('ctt_transfer_history_listing_view_log_message_other')
};

/**
 * @method _createView
 * @protected
 */
prototype._createView = function() {
  var v;
  var p;
  var s;

  superConstructor.prototype._createView.call(this);

  v = this._view;
  p = v.parent();
  s = TEMPLATE_SCROLLER.
      replace('${status}', _('ctt_transfer_history_listing_view_header_status')).
      replace('${date}',  _('ctt_transfer_history_listing_view_header_date')).
      replace('${title}',  _('ctt_transfer_history_listing_view_header_title')).
      replace('${destination}',  _('ctt_transfer_history_listing_view_header_destination'));
  v.detach();
  s = $(s).appendTo(p);
  $('.ctt-scroller-content', s).append(v);
  this._scroller = s;
};

/**
 * @method show
 */
prototype.show = function() {
  var s;
  var p;

  superConstructor.prototype.show.call(this);

  if (!!(s = this._scroller)) {
    p = s.parent();
    s.show();
    p.unbind('.TransferHistoryListingView');
    p.bind('resize.TransferHistoryListingView',
           $.proxy(this._didResizeParentContainer, this));
    this._adjustScrollerHeight();
  }
};

prototype._didResizeParentContainer = function() {
  this._adjustScrollerHeight();
};

prototype._adjustScrollerHeight = function() {
  var s = this._scroller;
  var c;
  var h;
  var p;

  if (!!!s) return;

  p = s.parent();
  s.height(p.height());

  c = $('.ctt-scroller-content', s);
  h = $('.ctt-scroller-header', s);
  c.height(Math.floor(s.height() - h.outerHeight(true)));
};

/**
 * @method hide
 */
prototype.hide = function() {
  superConstructor.prototype.hide.call(this);
  if (!!this._scroller) {
    this._scroller.hide();
  }
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  var s;

  superConstructor.prototype.destroy.call(this);
  if (!!(s = this._scroller)) {
    s.parent().unbind('.TransferHistoryListingView');
    s.remove();
  }
};

/**
 * @method _createRowElementHtml
 * @param {Object} record
 * @param {Number} index
 * @protected
 */
prototype._createRowElementHtml = function(record, index) {
  var id;
  var html;
  var status;
  var date;
  var wrap;
  var dest;

  wrap = function(c) {
    return TEMPLATE_CELL_BOUNDS_HARNESS.replace('${cellContent}', c);
  };

  if (!!!(id = this._makeElementId(record))) {
    return null;
  }

  status = LogStatus[record.status] || LogStatus['default'];
  date = Formatter.dcDateToDate(record.date);
  date = Formatter.stringifyDate(date) || '';
  dest = TEMPLATE_CELL_DESTINATION.
         replace('${server}', escapeHtml(record.server || '')).
         replace('${storage}', escapeHtml(record.storage || ''));

  html = TEMPLATE_ROW.
         replace('${id}', id).
         replace('${title}', wrap(escapeHtml(record.title || ''))).
         replace('${date}', wrap(date)).
         replace('${status}', wrap(status)).
         replace('${destination}', wrap(dest));

  return html;
};

}());
