/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule deletion-dialog-controller
 */

(function() {
var DeletionDialogController;
var prototype;
var superConstructor;
var DeletionDialogView = dixim.server.DeletionDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientServer = dixim.common.ApiClientServer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var Enum = dixim.ctt.util.Enum;

/**
 * @class DeletionDialogController
 * @constructor
 * @param {Object} configs
 */
DeletionDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(DeletionDialogController, EventTarget);
dixim.server.DeletionDialogController = DeletionDialogController;
prototype = DeletionDialogController.prototype;
superConstructor = DeletionDialogController.superConstructor;

var YES = 'yes';
var COMPLETE = 'complete';
var CLOSE = 'close';
var UPDATE = 'update';

var Mode = Enum.fetch('DELETION');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'deletion-dialog'].join('');

    this._mode = configs.mode;

    this._model = new ApiClientServer();
    switch(this._mode) {
        case Mode.SHARED_FOLDER:
            this._model.addEventListener('removeSharedFolder', this._success, this);
            this._model.addEventListener('removeSharedFolderError', this._error, this);
            break;
    };

    this._view = new DeletionDialogView(configs);
    this._view.addEventListener(YES, this._clickYes, this);
    this._view.addEventListener(COMPLETE, this._complete, this);

    this._contents = configs.contents;
    this._isProcessError;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._success = function() {
    this._view.decrementDescription();
};

prototype._error = function() {
    this._isProcessError = true;
    this._view.decrementDescription();
};

prototype._deleteContents = function() {
    var contentsLen = this._contents.length;
    var query = {};

    this._isProcessError = false;
    for (var i = 0; i < contentsLen; i += 1) {
        switch(this._mode) {
            case Mode.SHARED_FOLDER:
                query.sharedFolder = this._contents[i].path;
                this._model.removeSharedFolder(query);
                break;
        }
    }
};

prototype._complete = function() {
    this.stop();
    if (this._isProcessError) {
        this._errorDialog(_('server_public_folder_delete_error'));
        this.dispatchEvent(new Event(CLOSE));
    } else {
        this.dispatchEvent(new Event(UPDATE));
    }
};

prototype._clickYes = function() {
    this._deleteContents();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    switch(this._mode) {
        case Mode.SHARED_FOLDER:
            query.title = _('server_public_folder_delete_title');
            break;
    }
    query.id = this._id;
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
