/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule friendly-name-view
 */

(function() {
var FriendlyNameView;
var prototype;
var superConstructor;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var UiUtils = dixim.common.UiUtils;
var $ = dixim.ctt.jQuery;

/**
 * @class FriendlyNameView
 * @constructor
 * @param {Object} configs
 */
FriendlyNameView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(FriendlyNameView, EventTarget);
dixim.server.FriendlyNameView = FriendlyNameView;
prototype = FriendlyNameView.prototype;
superConstructor = FriendlyNameView.superConstructor;

var FORMAT = 'format';
var APPLY = 'apply';
var PREV = 'prev';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._title = _('server_menu_friendly_name');
    this._uiUtils = UiUtils;
};

/**
 * @method show
 */
prototype.show = function() {
    $('.rootPage').html([
        '<div class="contents" data-enhance="false">',
        '<h2 id="tit_page">', this._title, '</h2>',
        '<div class="contents_inner">',
        '<div class="dl_tbl">',
        '<dl>',
        '<dt id="friendly_name_title">', _('server_friendly_name_title'), '</dt>',
        '<dd>',
        '<input id="friendly_name_val" type="text" class="form_block" />',
        '</dd>',
        '</dl>',
        '</div>',
        '</div>',
        '</div>',
    ].join(''));

    this._uiUtils.cleanActionSettingBar();
    this._uiUtils.cleanActionBar();
    this._uiUtils.setActionBar('ab_format', dixim.proxy(this._clickFormat, this));
    this._uiUtils.setActionBar('ab_apply', dixim.proxy(this._clickApply, this));

    $('.art_tit').text(this._title);
    $('.btn_prev').show().one('click', dixim.proxy(this._clickPrev, this));
};

/**
 * @method hide
 */
prototype.hide = function() {
    $('.rootPage').empty();
};

prototype.setValue = function(name, isError) {
    $('#friendly_name_val').val(name);
    $('#friendly_name_val').prop('disabled', isError);
};

prototype._clickFormat = function() {
    this.dispatchEvent(new Event(FORMAT));
};

prototype._clickApply = function() {
    this.dispatchEvent(new Event(APPLY, {friendlyName: $('#friendly_name_val').val()}));
};

prototype._clickPrev = function() {
    this.dispatchEvent(new Event(PREV));
};

}());
