/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule media-type-dialog-view
 */

(function() {
var MediaTypeDialogView;
var prototype;
var superConstructor;
var ModalDialogView = dixim.common.ModalDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;
var Enum = dixim.ctt.util.Enum;

/**
 * @class MediaTypeDialogView
 * @constructor
 * @param {Object} configs
 */
MediaTypeDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(MediaTypeDialogView, ModalDialogView);
dixim.server.MediaTypeDialogView = MediaTypeDialogView;
prototype = MediaTypeDialogView.prototype;
superConstructor = MediaTypeDialogView.superConstructor;

var EDIT = 'edit';
var ADD = 'add';

var Mode = Enum.fetch('DIALOG');

var TEMPLATE_BODY_CONTENT = [
    '<div class="dlg_main_msg">', '${description}',
    '<div id="mediaTypeErrorField">', '</div>',
    '</div>',
    '<div class="dl_tbl">',
    '<dl class="none_border_bottom">',
    '<dt>', '${name}', '</dt>',
    '<dd>',
    '<div class="ui-input-text ui-body-inherit ui-corner-all ui-shadow-inset">',
    '<select id="mediaType" class="select-pointer" data-role="none">', '</select>',
    '</div>',
    '</dd>',
    '</dl>',
    '</div>',
].join('');
var MEDIA_TYPE_RANGE = {min: 1, max: 7};

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    configs.id = [configs.id, 'media-type-dialog-'].join('');

    this._mode = configs.mode;

    configs.bodyContent = TEMPLATE_BODY_CONTENT
        .replace('${name}', _('server_public_folder_edit_media_type_name'))
        .replace('${description}', _('server_public_folder_edit_description'));

    configs.buttons = [];

    switch(this._mode) {
        case Mode.ADD:
            configs.title = _('server_public_folder_add_title');
            configs.buttons.push({
                text: _('dialog_add'),
                click: dixim.proxy(this._clickAdd, this),
            });
            break;
        case Mode.EDIT:
            configs.title = _('server_public_folder_edit_title');
            configs.closeButton = dixim.proxy(this.hide, this);
            configs.buttons.push({
                text: _('dialog_cancel'),
                click: dixim.proxy(this.hide, this),
            });
            configs.buttons.push({
                text: _('dialog_edit'),
                click: dixim.proxy(this._clickEdit, this),
            });
            break;
    }

    this._sharedFolder = configs.sharedFolder;

    superConstructor.prototype._initialize.call(this, configs);
};

prototype._render = function() {
    for (var i = MEDIA_TYPE_RANGE['min']; i <= MEDIA_TYPE_RANGE['max']; i += 1) {
        $('#mediaType').append([
            '<option value="', i, '">', _(['server_public_folder_media_', i].join('')), '</option>',
        ].join(''));
    }
    $('#mediaType').val(this._sharedFolder.mediaType);
    superConstructor.prototype._render.call(this);
};

prototype.error = function(text) {
    $('#mediaTypeErrorField').html([
        '<span class="error-text">', text, '</span>',
    ].join(''));
};

prototype._clickEdit = function() {
    var mediaType = $('#mediaType').val();

    this.dispatchEvent(new Event(EDIT, {mediaType: mediaType}));
};

prototype._clickAdd = function() {
    var mediaType = $('#mediaType').val();

    this.dispatchEvent(new Event(ADD, {mediaType: mediaType}));
};

}());
